/*
  $Id: AnnotationData.java 6918 2015-05-26 06:43:54Z nicklas $

	Copyright (C) 2005 Nicklas Nordborg
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

  BASE is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 3
  of the License, or (at your option) any later version.

  BASE is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.Date;
import java.util.Set;

/**
	This class holds information about an annotation
	
	@author Nicklas
	@version 2.0
	@see net.sf.basedb.core.Annotation
	@see <a href="../../../../../../html/developer/api/data_api.annotations.html">Developer documentation: Annotations</a>
	@hibernate.class table="`Annotations`" lazy="false"
 */
public class AnnotationData
	extends BasicData
	implements LoggableData
{
	public AnnotationData()
	{}

	private AnnotationSetData annotationSet;
	/**
		Get the annotation set this annotation primarily belongs to.
		@hibernate.many-to-one outer-join="false" update="false"
		@hibernate.column name="`annotationset_id`" not-null="true" 
	*/
	public AnnotationSetData getAnnotationSet()
	{
		return annotationSet;
	}
	public void setAnnotationSet(AnnotationSetData annotationSet)
	{
		this.annotationSet = annotationSet;
	}

	private AnnotationTypeData annotationType;
	/**
		Get the type of this annotation.
		@hibernate.many-to-one outer-join="false" update="false"
		@hibernate.column name="`annotationtype_id`" not-null="true" 
	*/
	public AnnotationTypeData getAnnotationType()
	{
		return annotationType;
	}
	public void setAnnotationType(AnnotationTypeData annotationType)
	{
		this.annotationType = annotationType;
	}
	
	private int source;
	/**
		The annotation source.
		0 = primary, 1 = inherited, 2 = cloned

		For primary annotations, only one annotation per annotation type is supported.
		Inherited annotations should take values, units, etc. from the parent annotation
		Cloned annotations may have different values and units due to changes on the
		parent item or this item. The link to the parent item can be lost if the annotation
		is deleted from the parent.
		
		@since 3.6
		@hibernate.property column="`source`" type="int" not-null="true" update="false"
	*/
	public int getSource()
	{
		return source;
	}
	public void setSource(int source)
	{
		this.source = source;
	}
	
	private AnnotationData inheritedFrom;
	/**
		The parent annotation if this is an inherited or cloned annotation. Null otherwise.
		NOTE! Can be null also for cloned annotations if the parent annotation has been
		deleted.
		
		@hibernate.many-to-one column="`inherited_id`" not-null="false" update="false" outer-join="false" 
		@since 3.6
	*/
	public AnnotationData getInheritedFrom()
	{
		return inheritedFrom;
	}
	public void setInheritedFrom(AnnotationData inheritedFrom)
	{
		this.inheritedFrom = inheritedFrom;
	}
	
	private UnitData unit;
	/**
		Get the unit used for the annotation values. NOTE! The value
		is always null for annotation types that doesn't specify a
		quantity, and is always not null for annotation types that does.
		It is not possible to change between null and non-null.
		@hibernate.many-to-one column="`unit_id`" not-null="false" outer-join="false" 
		@since 2.9
	*/
	public UnitData getUnit()
	{
		return unit;
	}
	/**
		@since 2.9
	 */
	public void setUnit(UnitData unit)
	{
		this.unit = unit;
	}

	private ParameterValueData<?> values;
	/**
		The values of this annotation.
		@hibernate.many-to-one column="`value_id`" not-null="false" update="false" cascade="all" unique="true"
	*/
	public ParameterValueData<?> getValues()
	{
		return values;
	}
	public void setValues(ParameterValueData<?> values)
	{
		this.values = values;
	}
	
	private Date lastUpdate;
	/**
		Get the date and time the values in this annotation was last updated.
		@return A <code>Date</code> object, or null if not known
		@hibernate.property column="`last_update`" type="timestamp" not-null="false"
	*/
	public Date getLastUpdate()
	{
		return lastUpdate;
	}
	public void setLastUpdate(Date lastUpdate)
	{
		this.lastUpdate = lastUpdate;
	}
	
	private Set<AnnotationData> inheritingAnnotations;
	/**
		Other annotations which inherit this annotation. This is the inverse end.
		see {@link AnnotationData#getInheritedFrom()}
		@hibernate.set lazy="true" inverse="true" cascade="delete"
		@hibernate.collection-key column="`inherited_id`"
		@hibernate.collection-one-to-many class="net.sf.basedb.core.data.AnnotationData"
	*/
	Set<AnnotationData> getInheritingAnnotations()
	{
		return inheritingAnnotations;
	}
	void setInheritingAnnotations(Set<AnnotationData> inheritingAnnotations)
	{
		this.inheritingAnnotations = inheritingAnnotations;
	}


}
