/*
	$Id $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.data;

import java.util.List;

import net.sf.basedb.core.DateUtil;

import java.util.ArrayList;
import java.util.Date;

/**
	Date+time parameter value.

	@author Nicklas
	@since 2.16
	@see <a href="../../../../../../html/developer/api/data_api.parameters.html">Developer documentation: Parameters</a>
	@hibernate.subclass discriminator-value="11"
	@base.modified $Date $
*/
public class TimestampParameterValueData
	extends ParameterValueData<Date>
{
	public TimestampParameterValueData()
	{}
	
	public TimestampParameterValueData(Date... values)
	{
		super(values);
	}
	
	/**
		@return 9 (Type.TIMESTAMP)
		@since 2.16
	*/
	@Override
	public int getValueType()
	{
		return 9;
	}

	private List<Date> values;
	/**
		@hibernate.bag table="`TimestampValues`" lazy="true" cascade="all"
		@hibernate.collection-key column="`id`"
		@hibernate.collection-element column="`value`" type="timestamp" not-null="true"
	*/
	@Override
	public List<Date> getValues()
	{
		if (values == null )
		{
			values = new ArrayList<Date>();
		}
		return values;
	}
	@Override
	void setValues(List<Date> values)
	{
		this.values = values;
	}

	
	/**
		Overrides the parent method since we need date-specific equals
		implementation.
		@since 3.9.1
	*/
	@SuppressWarnings("unchecked")
	@Override
	public boolean replaceValuesIfDifferent(List<?> values)
	{
		List<Date> current = getValues();
		if (DateUtil.safeInvariantEquals((List<Date>)values, current, true)) return false;
		
		storeCurrentValuesAsOld(current);
		current.clear();
		if (values != null) current.addAll((List<Date>) values);
		return true;
	}
		
	/**
		Overrides the parent method since we need date-specific equals
		implementation.
		@since 3.9.1
	*/
	@Override
	public boolean setSingleValueIfDifferent(Object value)
	{
		List<Date> current = getValues();
		if (current != null && current.size() == 1 && DateUtil.safeEquals(current.get(0), (Date)value, true)) 
		{
			return false;
		}
	
		storeCurrentValuesAsOld(current);
		current.clear();
		current.add((Date)value);
		return true;
	}

	
	@Override
	public String toString()
	{
		return values.toString();
	}	
}
