/**
	$Id: PlateEventLogger.java 6497 2014-06-26 06:48:25Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.core.log.db;

import org.hibernate.Hibernate;
import org.hibernate.Query;

import net.sf.basedb.core.Item;
import net.sf.basedb.core.LogControl;
import net.sf.basedb.core.PredefinedQuery;
import net.sf.basedb.core.data.ChangeHistoryDetailData;
import net.sf.basedb.core.data.PlateEventData;
import net.sf.basedb.core.data.PlateEventTypeData;
import net.sf.basedb.core.log.EntityDetails;
import net.sf.basedb.core.log.ChangeType;
import net.sf.basedb.core.log.EntityLogger;
import net.sf.basedb.core.log.LogManager;

/**
	Logger for plate events that logs changes as
	updates to the plate the event belongs to.

	@author Nicklas
	@version 2.13
	@base.modified $Date: 2014-06-26 08:48:25 +0200 (to, 26 jun 2014) $
*/
public class PlateEventLogger
	implements EntityLogger
{

	private final DbLogManagerFactory factory;
	private final boolean detailedProperties;

	/**
		Creates a new plate event logger.
		@param factory The log manager factory currently in use
	*/
	public PlateEventLogger(DbLogManagerFactory factory)
	{
		this.factory = factory;
		this.detailedProperties = factory.isLoggingDetailedProperties();
	}
	
	/*
		From the EntityLogger interface
		-------------------------------
	*/
	@Override
	public void logChanges(LogManager logManager, EntityDetails details)
	{
		LogControl logControl = logManager.getLogControl();
		PlateEventData event = (PlateEventData)details.getEntity();
		
		// Get the name of the event type
		String eventType = null;
		PlateEventTypeData et = event.getPlateEventType();
		if (!Hibernate.isInitialized(et))
		{
			Query q = logControl.createHqlQuery(
					PredefinedQuery.getQueryString("DBLOG_GET_NAME_FOR_ITEM", "PlateEventTypeData"));
			q.setInteger("id", et.getId());
			eventType = (String)q.uniqueResult();
		}
		else
		{
			eventType = et.getName();
		}

		ChangeHistoryDetailData change = new ChangeHistoryDetailData();
		change.setChangeType(ChangeType.UPDATE.getValue());
		change.setItemId(event.getPlate().getId());
		change.setItemType(Item.PLATE.getValue());
		change.setChangeInfo(details.getChangeType().was("Event") + ": " + eventType);
		if (details.getChangeType() == ChangeType.UPDATE && detailedProperties)
		{
			String moreInfo = details.getModifiedProperties(null, ", ", null);
			if (moreInfo != null) 
			{
				change.setChangeInfo(change.getChangeInfo() + " (" + moreInfo + ")");
			}
		}
		((DbLogManager)logManager).logChangeDetails(change, details);
	}
	// -------------------------------
}
