/**
	$Id: MatrixBfsExporter.java 5319 2010-04-20 08:02:16Z nicklas $

	Copyright (C) 2009 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.export.spotdata;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import net.sf.basedb.core.Path;
import net.sf.basedb.util.bfs.DataWriter;

/**
	BFS exporter implementation that exports spot data in matrix format. This
	format creates one file per spot data field. Each data file contains
	data columns for all assays in the bioassay set. For example, the 'sdata1'
	file contains channel 1 intensity and 'sdata2' contains channel 2 intensity.
	
	@author Nicklas
	@version 2.15
	@base.modified $Date: 2010-04-20 10:02:16 +0200 (ti, 20 apr 2010) $
*/
public class MatrixBfsExporter
	extends BfsExporter
{
	
	public static final String SUBTYPE = "matrix";

	/**
		Create a new matrix BFS exporter.
	*/
	public MatrixBfsExporter()
	{}
	
	/*
		From the BfsExporter class
		--------------------------
	*/
	/**
		@return "matrix"
	*/
	@Override
	public String getBFSSubtype()
	{
		return SUBTYPE;
	}

	/**
		Create one data writer for each registered spot field. Each data 
		writer is configured with one column per assay in the bioassay set.
	*/
	@Override
	protected List<DataWriter> createDataWriters()
		throws IOException
	{
		prepareAssayIndexMap(getBioAssays(), 0, 1);
		List<DynamicField> spotFields = getSpotFields();
		int numAssays = getBioAssays().size();
		List<DataWriter> writers = new ArrayList<DataWriter>(spotFields.size());
		for (DynamicField df : spotFields)
		{
			DataWriter writer = createDataWriter(df, Path.makeSafeFilename(df.getTitle() + ".txt", ""));
			writer.setColumnCount(numAssays);
			writers.add(writer);
		}
		return writers;
	}

	/**
		The column number is used to find the assay index. Then,
		the spot data is copied to the correct data writer.
	*/
	@Override
	protected void arrangeData(int position, short column, 
		Object[][] data, Object[] spotData)
	{
		int columnToArrayIndex = getAssayIndex(column);
		for (int i = 0; i < data.length; ++i)
		{
			data[i][columnToArrayIndex] = spotData[i];
		}
	}
	// -------------------------------------------------------


}
