/**
	$Id $

	Copyright (C) 2010 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.formatter;

import net.sf.basedb.core.File;

/**
 	Formatter implementation that format {@link File} items
 	to their complete paths.
 	
  	@author Nicklas
  	@version 2.16
  	@base.modified $Date $
 */
public class FilePathFormatter 
	implements Formatter<File>
{
	private String nullValue = "";

	/**
		Create a new formatter. Null values are formatted as an empty string.
	*/
	public FilePathFormatter()
	{}
	
	/**
		Create a new formatter.
		@param nullValue The string to return if a null item is 
			passed to {@link #format(File)}
	*/
	public FilePathFormatter(String nullValue)
	{
		this.nullValue = nullValue;
	}
	
	/*
		From the Formatter interface
		-------------------------------------------
	*/
	@Override
	public String format(File file)
	{		
		return file == null ? nullValue : file.getPath().toString();
	}

	/**
		@throws UnsupportedOperationException Always
	*/
	@Override
	public File parseString(String value)
	{
		throw new UnsupportedOperationException("parseString: " + value);
	}
	// -------------------------------------------

}
