/**
	$Id: ConnectionManager.java 6497 2014-06-26 06:48:25Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.uri;

import java.io.IOException;
import java.io.InputStream;
import java.net.URI;

/**
	A connection manager is used to access the contents and metadata about
	the resource pointed to by a given URI. Connection managers are usually
	created by {@link ConnectionManagerFactory} objects. Implementations
	doesn't have to be thread-safe since a new connection manager is created
	for each use of an URI. Implementations doens't have to provide a public
	no-argument constructor.
	<p>
	NOTE! Also consider implementing the {@link ResumableConnectionManager}
	interface if the protocol supports setting a start offset for the
	download.

	@author Nicklas
	@since 3.0
	@base.modified $Date: 2014-06-26 08:48:25 +0200 (to, 26 jun 2014) $
*/
public interface ConnectionManager
{

	/**
		Get the URI of the resource we are interested in.
		@return An URI
	*/
	public URI getURI();
	
	/**
		Get an InputStream for reading the contents of the resource.
		@return An InputStream object, or null if the no data is available
		@throws IOException If there is an error creating the stream
	*/
	public InputStream getInputStream()
		throws IOException;

	/**
	 	Get metadata about the resource. If no metadata can be obtained
	 	the handler should still create an empty UriMetadata object.
		@return A metadata object
		@throws IOException If there is an error getting the metadata
	*/
	public UriMetadata getMetadata()
		throws IOException;
	
}

