/**
	$Id: UriMetadata.java 5582 2011-03-15 12:55:59Z nicklas $

	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
*/
package net.sf.basedb.util.uri;

import java.net.URI;
import java.util.Date;

/**
	Holds metadata about the data referenced by given URI. All metadata
	information is optional and clients should be prepared to handle
	it.

	@author Nicklas
	@since 3.0
	@base.modified $Date: 2011-03-15 13:55:59 +0100 (ti, 15 mar 2011) $
*/
public class UriMetadata
{

	private final URI uri;
	private Long length;
	private Date lastModified;
	private String mimeType;
	private String charset;
	private String md5;
	
	/**
		Create an empty metadata object for the given URI.
		@param uri The URI that defines the referenced resource
	*/
	public UriMetadata(URI uri)
	{
		this.uri = uri;
	}
	
	/**
		Get the length in bytes of the resource. 
		@return The size in bytes or null if not known
	*/
	public Long getLength()
	{
		return length;
	}
	public void setLength(Long length)
	{
		this.length = length;
	}
	
	/**
		Get the date and time the resource was last modified.
		@return A date or null if not known
	*/
	public Date getLastModified()
	{
		return lastModified;
	}
	public void setLastModified(Date lastModified)
	{
		this.lastModified = lastModified;
	}

	/**
		Get the MIME type of the resource.
		@return A string or null if not known
	*/
	public String getMimeType()
	{
		return mimeType;
	}
	public void setMimeType(String mimeType)
	{
		this.mimeType = mimeType;
	}
	
	/**
		Get the character set used by the resource.
		@return A string or null if not known
	*/
	public String getCharacterSet()
	{
		return charset;
	}
	public void setCharacterSet(String charset)
	{
		this.charset = charset;
	}
	
	/**
		Get the MD5 checksum of the resource.
		@return A string or null if not known
	*/
	public String getMd5()
	{
		return md5;
	}
	public void setMd5(String md5)
	{
		this.md5 = md5;
	}
	
	
}
